﻿///////////////////////////////////////////////////////////////////////////////
// Activity: program.activity0
// Processor type definitions
///////////////////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Xml;

using ccr = Microsoft.Ccr.Core;
using dss = Microsoft.Dss.Core;
using dssa = Microsoft.Dss.Core.Attributes;
using dssh = Microsoft.Dss.Core.DsspHttp;
using dssm = Microsoft.Dss.ServiceModel.DsspServiceBase;
using dssp = Microsoft.Dss.ServiceModel.Dssp;
using soap = W3C.Soap;

using submgr = Microsoft.Dss.Services.SubscriptionManager;
using visionclient = Robotics.Vision.VisionClient.Proxy;

[assembly: dssa.ServiceDeclaration(dssa.DssServiceDeclaration.ServiceBehavior)]
[assembly: System.Runtime.InteropServices.ComVisible(false)]

namespace Robotics.Blob.Processor
{
    static class Contract
    {
        public const string Identifier = "http://schemas.tempuri.org/2007/10/blob/processor.html";
    }

    [dssa.DataContract]
    public class ProcessorState
    {
    }

    #region Custom message types

    [dssa.DataContract]
    public class processRequest
    {
        List<visionclient.Blob> _blobs;
        [dssa.DataMember]
        public List<visionclient.Blob> Blobs
        {
            get { return _blobs; }
            set { _blobs = value; }
        }
    }

    [dssa.DataContract]
    public class processResponse
    {
        double _leftSpeed;
        [dssa.DataMember]
        public double LeftSpeed
        {
            get { return _leftSpeed; }
            set { _leftSpeed = value; }
        }

        double _rightSpeed;
        [dssa.DataMember]
        public double RightSpeed
        {
            get { return _rightSpeed; }
            set { _rightSpeed = value; }
        }
    }

    #endregion

    #region Operations Port

    public class ProcessorOperations : ccr.PortSet
    {
        public ProcessorOperations() : base (
            typeof(dssp.DsspDefaultLookup),
            typeof(dssp.DsspDefaultDrop),
            typeof(dssh.HttpGet),
            typeof(process),
            typeof(Get),
            typeof(Replace),
            typeof(Subscribe)
        )
        {
        }

        public static implicit operator ccr.Port<dssp.DsspDefaultLookup>(ProcessorOperations portSet)
        {
            if (portSet == null)
            {
                return null;
            }
            return (ccr.Port<dssp.DsspDefaultLookup>)portSet[typeof(dssp.DsspDefaultLookup)];
        }

        public void Post(dssp.DsspDefaultLookup msg)
        {
            base.PostUnknownType(msg);
        }

        public static implicit operator ccr.Port<dssp.DsspDefaultDrop>(ProcessorOperations portSet)
        {
            if (portSet == null)
            {
                return null;
            }
            return (ccr.Port<dssp.DsspDefaultDrop>)portSet[typeof(dssp.DsspDefaultDrop)];
        }

        public void Post(dssp.DsspDefaultDrop msg)
        {
            base.PostUnknownType(msg);
        }

        public static implicit operator ccr.Port<Get>(ProcessorOperations portSet)
        {
            if (portSet == null)
            {
                return null;
            }
            return (ccr.Port<Get>)portSet[typeof(Get)];
        }

        public void Post(Get msg)
        {
            base.PostUnknownType(msg);
        }

        public static implicit operator ccr.Port<Replace>(ProcessorOperations portSet)
        {
            if (portSet == null)
            {
                return null;
            }
            return (ccr.Port<Replace>)portSet[typeof(Replace)];
        }

        public void Post(Replace msg)
        {
            base.PostUnknownType(msg);
        }

        public static implicit operator ccr.Port<Subscribe>(ProcessorOperations portSet)
        {
            if (portSet == null)
            {
                return null;
            }
            return (ccr.Port<Subscribe>)portSet[typeof(Subscribe)];
        }

        public void Post(Subscribe msg)
        {
            base.PostUnknownType(msg);
        }

        public static implicit operator ccr.Port<process>(ProcessorOperations portSet)
        {
            if (portSet == null)
            {
                return null;
            }
            return (ccr.Port<process>)portSet[typeof(process)];
        }

        public void Post(process msg)
        {
            base.PostUnknownType(msg);
        }

        public dssp.DsspResponsePort<processResponse> process(processRequest body)
        {
            process message = new process();
            message.Body = body;

            this.Post(message);

            return message.ResponsePort;
        }

        public dssp.DsspResponsePort<dssp.SubscribeResponseType> Subscribe(ccr.IPort notificationPort)
        {
            Subscribe message = new Subscribe();
            message.NotificationPort = notificationPort;

            this.Post(message);

            return message.ResponsePort;
        }
    }

    #endregion

    #region Standard DSSP message definitions

    [Description("Retrieves the current state of the service")]
    public class Get : dssp.Get<dssp.GetRequestType, dssp.DsspResponsePort<ProcessorState>>
    {
    }

    [Description("Sets the current state of the service.\nThis is raised as an event whenever the state changes.")]
    public class Replace : dssp.Replace<ProcessorState, dssp.DsspResponsePort<dssp.DefaultReplaceResponseType>>
    {
    }

    public class Subscribe : dssp.Subscribe<dssp.SubscribeRequestType, dssp.DsspResponsePort<dssp.SubscribeResponseType>>
    {
    }

    #endregion

    #region Custom message definitions

    public class process : dssp.Submit<processRequest, dssp.DsspResponsePort<processResponse>>
    {
    }

    #endregion
}
